/*
    *
    * This file is a part of CoreStuff.
    * An activity viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include "pageapps.h"
#include "ui_pageapps.h"

#include <QtConcurrent/QtConcurrent>
#include <QFuture>
#include <QGraphicsDropShadowEffect>
#include <QtEvents>
#include <QScrollBar>

#include <cprime/corexdg.h>
#include <cprime/applicationdialog.h>
#include <cprime/appopenfunc.h>

QMutex appLock;

pageapps::pageapps(QWidget *parent) :QWidget(parent),ui(new Ui::pageapps)
{
    ui->setupUi(this);
    loadSettings();
    startSetup();
    setupAppsView();
}

pageapps::~pageapps()
{
    delete ui;
}

/**
 * @brief Loads application settings
 */
void pageapps::loadSettings()
{
    iconViewIconSize = sm->value("CoreApps", "IconViewIconSize");
}
void pageapps::startSetup()
{
    ui->appsView->setIconSize( iconViewIconSize );
    ui->appsView->setGridSize( QSize( iconViewIconSize.width() * 3, iconViewIconSize.height() * 2 ) );

    QPalette pltt( palette() );
    QColor txtColor( pltt.color( QPalette::Text ) );

    QGraphicsDropShadowEffect *effect = new QGraphicsDropShadowEffect(this);
    effect->setBlurRadius( 0 );
    effect->setColor( QColor( 0xff - txtColor.red(), 0xff - txtColor.green(), 0xff - txtColor.blue() ) );
    effect->setOffset( 1, 1 );

    ui->appsView->setGraphicsEffect(effect);

    QFileSystemWatcher *watcher = new QFileSystemWatcher();
    watcher->addPath( "/usr/share/applications/" );

    connect( watcher, &QFileSystemWatcher::directoryChanged, this, &pageapps::setupAppsView );
}

void pageapps::setupAppsView()
{
	appLock.lock();
    ui->appsView->clear();

    QStringList appList;

	CoreXdgMime::instance()->parseDesktops();
    AppsList list = CoreXdgMime::instance()->allDesktops();

    std::sort( list.begin(), list.end(), [ ]( const CoreDesktopFile& lhs, const CoreDesktopFile& rhs ) {
        return lhs.name() < rhs.name();
    } );

    QFuture<void> f = QtConcurrent::run( [this, list, appList] {
        foreach ( CoreDesktopFile df, list ) {

            if ( not df.isValid() )
                continue;

            if ( not df.visible() )
                continue;

            QIcon icon = CPrime::ThemeFunc::getAppIcon( df.desktopName() );
            icon = CPrime::ThemeFunc::resizeIcon( icon, ui->appsView->iconSize() );

            QListWidgetItem *item = new QListWidgetItem( icon, df.name() );
            item->setData( Qt::UserRole + 1, QVariant::fromValue<CoreDesktopFile>( df ) );
            item->setSizeHint( QSize( 110, 100 ) );
            ui->appsView->addItem( item );
        }
    } );

    QFutureWatcher<void> *fw = new QFutureWatcher<void>();
    fw->setFuture( f );
    connect( fw, &QFutureWatcher<void>::finished, [&]() {
        ui->appsView->setSizeAdjustPolicy( QListWidget::AdjustToContents );
    } );

    appLock.unlock();
}

void pageapps::on_appsView_itemClicked(QListWidgetItem *item)
{
    CoreDesktopFile df = qvariant_cast<CoreDesktopFile>( item->data( Qt::UserRole + 1 ) );
    if ( df.isValid() )
        df.startApplicationWithArgs( QStringList() );
}


bool pageapps::handleMouseEvents( QMouseEvent *mEvent ) {

    if ( mEvent->type() == QEvent::MouseButtonPress ) {

        /* Scroll only the appsView */
        if ( ui->appsView->underMouse() )
            mStart = QCursor::pos();			// Lets use the global position.

        return false;
    }

    else if ( mEvent->type() == QEvent::MouseMove ) {

        if ( moveTimer.isActive() )
            return true;

        /* If the click and hold happened in appsView @mStart will be set */
        if ( not mStart.isNull() ) {
            /* Start the scrolling process */
            moveTimer.start( 50, this );

            return true;
        }

        return false;
    }

    else if ( mEvent->type() == QEvent::MouseButtonRelease ) {

        mStart = QPoint();				// Set the mStart to null point
        moveTimer.stop();				// Stop the scroll process

        /* If a scroll was performed, ignore everything else */
        if ( mScrolled ) {
            mScrolled = false;
            return true;
        }

        return false;
    }

    return false;
}

void pageapps::timerEvent( QTimerEvent *tEvent ) {

    if ( tEvent->timerId() == moveTimer.timerId() ) {

        if ( not mStart.isNull() ) {
            QPoint end = QCursor::pos();					// mStart also uses global value.

            int delta = end.y() - mStart.y();

            if ( delta ) {
                QScrollBar *vScroll = ui->appsView->verticalScrollBar();

                mScrolled = true;

                if ( delta > 0 )
                    vScroll->setSliderPosition( vScroll->sliderPosition() - 20 );

                else if ( delta < 0 )
                    vScroll->setSliderPosition( vScroll->sliderPosition() + 20 );
            }
        }
    }

    else
        QWidget::timerEvent( tEvent );
}
