/*
	*
	* This file is a part of CoreStuff.
	* An activity viewer for CuboCore Application Suite
	* Copyright 2019 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QDebug>
#include <QScreen>
#include <QPainter>
#include <QFileInfo>
#include <QResizeEvent>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QShortcut>
#include <QEvent>
#include <QMouseEvent>
#include <QMessageBox>

#include "global.h"

#include "pageactivites.h"
#include "apps/pageapps.h"
#include "pagehome.h"
#include "pagepins.h"
#include "pagesessions.h"
#include "pagetasks.h"

#include "slidingstackedwidget.h"
#include "corestuff.h"
#include "ui_corestuff.h"
#include "powerdlg.h"

#include <cprime/cprime.h>


corestuff::corestuff( QWidget *parent ):QWidget( parent ),ui( new Ui::corestuff )
{
	ui->setupUi( this );

	QPalette pltt( palette() );
	pltt.setColor( QPalette::Base, Qt::transparent );
	setPalette( pltt );

	loadSettings();
	startSetup();
	shortcuts();

	pHome = new pagehome( this );
	pApps = new pageapps( this );
	pTasks = new pagetasks( this );

	connect( pTasks, &pagetasks::resizeDesktop, this, &corestuff::resizeDesktop );
	connect( pTasks, SIGNAL( activateDock() ), this, SIGNAL( activateDock() ) );

	ui->pHomeLayout->addWidget(pHome);
	ui->pAppsLayout->addWidget(pApps);
    ui->pTasksLayout->addWidget(pTasks);

	// Set coreapps page as active page
	on_home_clicked();

	ui->menu->setVisible( 0 );
	ui->addSession->setVisible( 0 );
	ui->deleteSession->setVisible( 0 );
	ui->editSessoion->setVisible( 0 );
	ui->clearActivites->setVisible( 0 );

	qApp->installEventFilter( this );
}

corestuff::~corestuff()
{
	delete ui;
}

void corestuff::closeEvent(QCloseEvent *event)
{
	event->ignore();
	for (int i = 0; i < ui->pages->count(); i++) {
		if (ui->pages->widget(i)->children().count() == 2)
			if (ui->pages->widget(i)->children().at(1))
				ui->pages->widget(i)->children().at(1)->deleteLater();
	}
	event->accept();
}

void corestuff::startSetup()
{
	ui->user->setText(qgetenv("USER"));

	// corestuff as desktop
	if (desktop) {
		setFixedSize( qApp->primaryScreen()->size() );
		//setWindowState( Qt::WindowFullScreen );

		setAttribute( Qt::WA_X11NetWmWindowTypeDesktop );
		setWindowFlags( Qt::FramelessWindowHint );

		bgImage = QImage( QString( sm->value( "CoreStuff", "Background" ) ) ).scaled( qApp->primaryScreen()->size(), Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation );

	}

	// corestuff as app in full screen
	else {
		setWindowFlags(Qt::Window | Qt::FramelessWindowHint);
		this->setWindowState( Qt::WindowMaximized );
		qDebug ()<< "'tyry";
	}

    // icon only pagebuttons
    for (QToolButton *b : ui->pageButtons->findChildren<QToolButton *>()) {
        b->setToolButtonStyle(Qt::ToolButtonIconOnly);
        b->setIconSize(toolBarIconSize);
    }

	if(!showActivities)
		ui->activites->setVisible(false);

	fswStart = new QFileSystemWatcher( this );
	connect( fswStart, &QFileSystemWatcher::fileChanged, [ = ]( const QString & path ) {
		reload( path );
	} );

	fswStart->addPaths( QStringList() << CPrime::Variables::CC_CoreApps_RecentActFilePath()
						<< CPrime::Variables::CC_CoreApps_PinsFilePath()
						<< CPrime::Variables::CC_CoreApps_SessionFilePath());
}

void corestuff::shortcuts()
{
	QShortcut *shortcut;
	shortcut = new QShortcut(QKeySequence(Qt::Key_Left), this);
	connect(shortcut, &QShortcut::activated, this, &corestuff::previousPage);

	shortcut = new QShortcut(QKeySequence(Qt::Key_Right), this);
	connect(shortcut, &QShortcut::activated, this, &corestuff::nextPage);
}

void corestuff::previousPage()
{
	ui->pages->slideInPrev();
}

void corestuff::nextPage()
{
	ui->pages->slideInNext();
}

void corestuff::reload( const QString &path )
{
	QFileInfo fi( path );

	if ( fi.fileName() == CPrime::Variables::CC_CoreApps_RecentActFile() ) {
		if (showActivities) {
			if (pActivities) {
				pActivities->reload();
			}
		}
	}

	else if ( fi.fileName() == CPrime::Variables::CC_CoreApps_PinsFile() ) {
		if (pPins) {
			pPins->reload();
		}
	}

	else if ( fi.fileName() == CPrime::Variables::CC_CoreApps_SessionFile() ) {
		if (pSessions) {
			pSessions->reload();
		}
	}
}

/**
 * @brief Loads application settings
 */
void corestuff::loadSettings()
{
	touch = sm->value("CoreApps", "TouchMode");
	desktop = sm->value("CoreStuff", "Desktop");
	showActivities = sm->value("CoreApps", "KeepActivities");
	toolBarIconSize = sm->value("CoreApps", "ToolBarIconSize");
}

void corestuff::pageClick( QToolButton *btn, int i, QString windowTitle )
{
	// all button checked false
	for ( QToolButton *b : ui->pageButtons->findChildren<QToolButton *>() ) {
		b->setChecked( false );
	}

	btn->setChecked( true );
	ui->pages->slideInIdx( i );
	currentPage = i;
	setWindowTitle( windowTitle + " - CoreStuff" );
}

void corestuff::on_home_clicked()
{
	pageClick( ui->home, 0, "Home" );
	ui->menu->setVisible(0);
	ui->addSession->setVisible( 0 );
	ui->deleteSession->setVisible( 0 );
	ui->editSessoion->setVisible( 0 );
	ui->clearActivites->setVisible( 0 );
}

void corestuff::on_apps_clicked()
{
	pageClick( ui->apps, 1, "Apps" );
	ui->menu->setVisible(0);
	ui->addSession->setVisible( 0 );
	ui->deleteSession->setVisible( 0 );
	ui->editSessoion->setVisible( 0 );
	ui->clearActivites->setVisible( 0 );
}

void corestuff::on_tasks_clicked()
{
	pTasks->updateClientList();
	pageClick( ui->tasks, 2, "Tasks" );
	ui->menu->setVisible(0);
	ui->addSession->setVisible( 0 );
	ui->deleteSession->setVisible( 0 );
	ui->editSessoion->setVisible( 0 );
	ui->clearActivites->setVisible( 0 );
}

void corestuff::on_pins_clicked()
{
	if (ui->pages->widget(3)->children().count() < 2) {
		pPins = new pagepins(this);
		ui->pPinsLayout->addWidget(pPins);
	}

	pageClick( ui->pins, 3, "Pins" );
	ui->menu->setVisible(0);
	ui->addSession->setVisible( 0 );
	ui->deleteSession->setVisible( 0 );
	ui->editSessoion->setVisible( 0 );
	ui->clearActivites->setVisible( 0 );
}


void corestuff::on_activites_clicked()
{
	if (ui->pages->widget(4)->children().count() < 2) {
		pActivities = new pageactivites(this);
		QList<QToolButton*> list;
		list.append(ui->menu);
		list.append(ui->clearActivites);
		pActivities->addControls(list);
		ui->pActivitesLayout->addWidget(pActivities);

		// Configure Recent Activity
		if (showActivities) {
			QString raFile = CPrime::Variables::CC_CoreApps_RecentActFilePath();
			QFile file( raFile );

			if ( !file.exists() ) {
				// You can get error
				// Need a check here
				file.open( QIODevice::ReadWrite | QIODevice::Text );
				file.close();
			}
		}
	}

	pageClick( ui->activites, 4, "Activites" );
	ui->menu->setVisible(true);
}

void corestuff::on_session_clicked()
{
	if (ui->pages->widget(5)->children().count() < 2) {
		pSessions = new pagesessions(this);
		QList<QToolButton*> list;
		list.append(ui->menu);
		list.append(ui->addSession);
		list.append(ui->editSessoion);
		list.append(ui->deleteSession);
		pSessions->addControls(list);
		ui->pSessionsLayout->addWidget(pSessions);
	}

	pageClick( ui->session, 5, "Sessions" );
	ui->menu->setVisible(true);
}

void corestuff::on_clearActivites_clicked()
{
	// link to pageactivites.cpp
	pActivities->clearActivities();
}

void corestuff::on_editSessoion_clicked()
{
	//link to pagesession.cpp
	pSessions->editSessoion();
}

void corestuff::on_deleteSession_clicked()
{
	//link to pagesession.cpp
	pSessions->deleteSession();
}

void corestuff::on_addSession_clicked()
{
	//link to pagesession.cpp
	pSessions->addSession();
}

void corestuff::on_menu_clicked()
{
	if ( currentPage == 4 ) { // activites
		ui->addSession->setVisible( 0 );
		ui->deleteSession->setVisible( 0 );
		ui->editSessoion->setVisible( 0 );
		ui->clearActivites->setVisible( 1 );
	}

	else if ( currentPage == 5 ) { // session
		ui->addSession->setVisible( 1 );
		ui->deleteSession->setVisible( 0 );
		ui->editSessoion->setVisible( 0 );
		ui->clearActivites->setVisible( 0 );
	}
}

void corestuff::handleMessages( const QString msg ) {

	// This is basically to reload the background
	if ( msg == "reload" ) {
		bgImage = QImage( sm->value( "CoreStuff", "Background" ) ).scaled( bgImage.size(), Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation );
		repaint();
	}
}

void corestuff::paintEvent( QPaintEvent *pEvent ) {

	if ( desktop ) {

		QPainter painter( this );
		painter.drawImage( QRect( QPoint( 0, 0 ), bgImage.size() ), bgImage );
		painter.end();
	}

	QWidget::paintEvent( pEvent );
	pEvent->accept();
};

void corestuff::showTasksPage() {

	on_tasks_clicked();
}

void corestuff::showDesktopT() {

	pTasks->showDesktop();
}

bool corestuff::eventFilter( QObject *obj, QEvent *event ) {

	Q_UNUSED( obj )

	if( event->type() == QEvent::MouseButtonPress ) {

		QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
		if ( pHome->underMouse() )
			return pHome->handleMouseEvents( mouseEvent );

		else if ( pApps->underMouse() )
			return pApps->handleMouseEvents( mouseEvent );

		return false;
	}

	else if( event->type() == QEvent::MouseMove ) {

		QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
		if ( pHome->underMouse() )
			return pHome->handleMouseEvents( mouseEvent );

		else if ( pApps->underMouse() )
			return pApps->handleMouseEvents( mouseEvent );

		return false;
	}

	else if( event->type() == QEvent::MouseButtonRelease ) {

		QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
		if ( pHome->underMouse() )
			return pHome->handleMouseEvents( mouseEvent );

		else if ( pApps->underMouse() )
			return pApps->handleMouseEvents( mouseEvent );

		return false;
	}

	return false;
}

void corestuff::on_power_clicked()
{
	PowerDialog *pwrDlg = new PowerDialog( this );
	pwrDlg->exec();
}

void corestuff::resizeDesktop() {

	Qt::ScreenOrientation scrOrient = qApp->primaryScreen()->orientation();
	QSize size = qApp->primaryScreen()->size();

	qDebug() << scrOrient;

	switch( scrOrient ) {
		case Qt::PrimaryOrientation: {

			setFixedSize( size );
			break;
		}

		case Qt::LandscapeOrientation:
		case Qt::InvertedLandscapeOrientation: {

			if ( size.width() > size.height() ) {
				setFixedSize( size.width(), size.height() );
				qDebug() << size.width() << size.height();
			}

			else {
				setFixedSize( size.height(), size.width() );
				qDebug() << size.height() << size.width();
			}

			break;
		}


		case Qt::PortraitOrientation:
		case Qt::InvertedPortraitOrientation: {

			if ( size.width() > size.height() )
				setFixedSize( size.height(), size.width() );

			else
				setFixedSize( size.width(), size.height() );

			break;
		}
	}
};
