/*
    *
    * This file is a part of CoreStuff.
    * An activity viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QPainter>
#include <QDebug>
#include <QScreen>
#include <QTimer>
#include <QPluginLoader>
#include <QDateTime>
#include <QMouseEvent>

#include <cprime/cplugininterface.h>
#include <cprime/cprime.h>
#include <cprime/corexdg.h>

#include "pagehome.h"
#include "ui_pagehome.h"


pagehome::pagehome(QWidget *parent) :QWidget(parent),ui(new Ui::pagehome)
{
    ui->setupUi(this);

    loadSettings();
    setupWidgetView();
}

pagehome::~pagehome()
{
    delete ui;
}

/**
 * @brief Loads application settings
 */
void pagehome::loadSettings()
{
    selectedPlugins = sm->value("CoreAction", "SelectedPlugins");
}


void pagehome::setupWidgetView()
{

    bool setOrientation = false ;

    for (int i = 0; i < qApp->screens().count(); i++) {

        if (qApp->screens()[i]->primaryOrientation() == Qt::LandscapeOrientation) {
            setOrientation = 0 ;  // Landscape
        } else {
            setOrientation = 1 ;  // Portrait
        }
    }

    if(!setOrientation){
        // set window size
        QScreen *scr = QGuiApplication::primaryScreen();
        int width = scr->availableGeometry().width();
        int x = static_cast<int>(width * .23);

        ui->widgetsL->setFixedWidth(x - 33);
        ui->widgets->setFixedWidth(x - 4);
        ui->frame->setFixedWidth(x - 6);
    } else{
        ui->sp->changeSize(QSizePolicy::Fixed, QSizePolicy::Expanding);
    }


    updateTime();
    QTimer *timer = new QTimer( this );
    connect( timer, &QTimer::timeout, this, &pagehome::updateTime );
    timer->start( 60000 );

    /* All Plugins */
    QStringList pluginPaths = selectedPlugins;
    if ( not pluginPaths.count() ) {
        CPrime::InfoFunc::messageEngine(
            "CoreStuff", "corestuff", "Plugins not found",
            "Please install plugins and ensure they are enabled in CoreGarage.",
            this
        );
    }

    Q_FOREACH( QString pPath, pluginPaths ) {

        QPluginLoader loader(pPath);
        QObject *pObject = loader.instance();
        if (pObject) {
            plugininterface *plugin = qobject_cast<plugininterface*>(pObject);
            if ( !plugin ) {
                qWarning() << "Plugin Error:" << loader.errorString();
                continue;
            }

            QWidget *w = plugin->widget( this );
            ui->widgetLayout->insertWidget( 0, w );
        } else {
            qWarning() << "Plugin Error:" << loader.errorString();
        }
    }
}

void pagehome::updateTime()
{
    QDateTime dt = QDateTime::currentDateTime();
    ui->day->setText( dt.toString( "ddd MMMM d " ));
    ui->time->setText(dt.toString("hh:mm AP"));
}

bool pagehome::handleMouseEvents( QMouseEvent *mEvent ) {

    if ( mEvent->type() == QEvent::MouseButtonPress ) {

        /* Scroll only the appsView */
        if ( ui->widgets->underMouse() )
            mStart = QCursor::pos();			// Lets use the global position.

        return false;
    }

    else if ( mEvent->type() == QEvent::MouseMove ) {

        if ( moveTimer.isActive() )
            return true;

        /* If the click and hold happened in appsView @mStart will be set */
        if ( not mStart.isNull() ) {
            /* Start the scrolling process */
            moveTimer.start( 50, this );

            return true;
        }

        return false;
    }

    else if ( mEvent->type() == QEvent::MouseButtonRelease ) {

        mStart = QPoint();				// Set the mStart to null point
        moveTimer.stop();				// Stop the scroll process

        /* If a scroll was performed, ignore everything else */
        if ( mScrolled ) {
            mScrolled = false;
            return true;
        }

        return false;
    }

    return false;
}

void pagehome::timerEvent( QTimerEvent *tEvent ) {

    if ( tEvent->timerId() == moveTimer.timerId() ) {

        if ( not mStart.isNull() ) {
            QPoint end = QCursor::pos();					// mStart also uses global value.

            int delta = end.y() - mStart.y();

            if ( delta ) {
                QScrollBar *vScroll = ui->widgets->verticalScrollBar();

                mScrolled = true;

                if ( delta > 0 )
                    vScroll->setSliderPosition( vScroll->sliderPosition() - 20 );

                else if ( delta < 0 )
                    vScroll->setSliderPosition( vScroll->sliderPosition() + 20 );
            }
        }
    }

    else
        QWidget::timerEvent( tEvent );
}
